/***********************************************************************
* Copyright by Michael Loesler, https://software.applied-geodesy.org   *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 3 of the License, or    *
* at your option any later version.                                    *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, see <http://www.gnu.org/licenses/>  *
* or write to the                                                      *
* Free Software Foundation, Inc.,                                      *
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
*                                                                      *
***********************************************************************/


package org.applied_geodesy.util.filesplitter.io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.applied_geodesy.util.io.SourceFileReader;

public class FlatFileSplitter extends SourceFileReader<Boolean> {
	private List<String> lines = null;
	private String lastStationName = null;
	private int fileIdx = 0;
	
	public FlatFileSplitter(String fileName) {
		this(new File(fileName).toPath());
	}

	public FlatFileSplitter(File sf) {
		this(sf.toPath());
	}
	
	public FlatFileSplitter(Path path) {
		super(path);
		this.setPath(path);
	}
	
	@Override
	public Boolean readAndImport() throws Exception {
		this.reset();
		this.fileIdx = 0;
		
		this.ignoreLinesWhichStartWith("#");
		
		super.read();
		
		// write last lines to file
		if (!this.lines.isEmpty())
			this.writeFile();
		
		this.reset();
		this.fileIdx = 0;
		
		return null;
	}

	@Override
	public void reset() {
		if (this.lines == null)
			this.lines = new ArrayList<String>();
		
		this.lastStationName = null;
		this.lines.clear();
	}

	@Override
	public void parse(String line) throws SQLException {
		String data[] = line.trim().split("\\s+");
		// at least station id, target id and an observed value
		if (data.length < 3)
			return;
		
		if (this.lastStationName != null && !data[0].equals(this.lastStationName)) {
			this.writeFile();
			this.reset();
		}
		
		if (this.lastStationName == null)
			this.lastStationName = data[0];
		
		this.lines.add(line);
		
	}

	private void writeFile() throws SQLException {
		
		Path path = this.getPath();
		Path dir  = path.getParent();
		Path file = path.getFileName();
		Path pathToFile = Paths.get(dir.toString(), String.valueOf(++this.fileIdx) + "_" + this.lastStationName.replaceAll("[^a-zA-Z0-9_\\.\\-]", "") + "_" + file.toString());
		
		PrintWriter pw = null;
    	try {
    		pw = new PrintWriter(new BufferedWriter(new FileWriter( pathToFile.toFile() )));
    		
    		for (String line : this.lines)
    			pw.println(line);
    		
    	} catch (IOException e) {
    		throw new SQLException(e);
    	}
    	finally {
    		if (pw != null) {
    			pw.close();
    		}
    	}
	}

}
